Summary: A collection of tools for managing UNIX services
Name: daemontools
Version: 0.76
Release: 1%{?dist}
License: See djb@pobox.com
Group: Utilities/System
Source0: http://cr.yp.to/daemontools/daemontools-%{version}.tar.gz
BuildRoot: /tmp/daemontools-buildroot
URL: http://cr.yp.to/daemontools.html
Packager: John Simpson <jms1@voalte.com>

%description
daemontools is a collection of tools for managing UNIX services. 

supervise monitors a service. It starts the service and restarts the
service if it dies. Setting up a new service is easy: all supervise
needs is a directory with a run script that runs the service.

multilog saves error messages to one or more logs. It optionally
timestamps each line and, for each log, includes or excludes lines
matching specified patterns. It automatically rotates logs to limit the
amount of disk space used. If the disk fills up, it pauses and tries
again, without losing any data.

%prep
%setup -n admin/daemontools-0.76/src

echo gcc %{optflags} -I. >conf-cc
echo gcc %{optflags} -I. -s >conf-ld
echo %{buildroot}/usr/local >home

mkdir sys
echo '#include </usr/include/sys/time.h>' >sys/time.h
echo '#include </usr/include/time.h>' >>sys/time.h

sed -e 's/extern int errno;/#include <errno.h>/' <error.h >error.h.tmp
mv error.h.tmp error.h

%build
make

%install
rm -fr %{buildroot}
mkdir -p %{buildroot}/usr/local/bin
while read command; do
	install -m 0755 $command %{buildroot}/usr/local/bin/$command
done <../package/commands

%clean
rm -rf %{buildroot}

%post
###############################################################################
#
# post-install script

########################################
# Check /service directory

if [[ -d /service ]]
then
    echo '/service already exists'
else
    echo 'Creating /service'
    mkdir -m 0755 /service
fi

########################################
# upstart in /etc/init (CentOS 6, others)

if [[ -d /etc/init ]]
then
    echo 'Found upstart in /etc/init'

    if [[ -f /etc/init/daemontools.conf ]]
    then
        echo '/etc/init/daemontools.conf already exists'
    else
        echo 'Adding daemontools to upstart'
        cat > /etc/init/daemontools.conf <<EOF
# upstart config for daemontools
start on runlevel [345]
stop on runlevel [!345]
respawn
exec /usr/local/bin/svscanboot
EOF
        chmod 0644 /etc/init/daemontools.conf
    fi

    echo 'Starting daemontools'
    /sbin/initctl start daemontools

########################################
# upstart in /etc/event.d (Fedora 12, others)

elif [[ -d /etc/event.d ]]
then
    echo "Found upstart in /etc/event.d"

    if [[ -f /etc/event.d/daemontools.conf ]]
    then
        echo '/etc/event.d/daemontools.conf already exists'
    else
        echo 'Adding daemontools to upstart'
        cat > /etc/event.d/daemontools.conf <<EOF
# upstart config for daemontools
start on runlevel [345]
stop on runlevel [!345]
respawn
exec /usr/local/bin/svscanboot
EOF
        chmod 0644 /etc/event.d/daemontools.conf
    fi

    echo 'Starting daemontools'
    /sbin/initctl start daemontools

########################################
# inittab (most others)

elif [[ -r /etc/inittab ]]
then
    echo 'Found /etc/inittab'

    if grep svscanboot /etc/inittab >/dev/null
    then
        echo 'inittab contains an svscanboot line. I assume that svscan is already running.'
    else
        echo 'Adding svscanboot to inittab...'
        rm -f /etc/inittab'{new}'
        cp /etc/inittab /etc/inittab'{new}'
        cat >> /etc/inittab'{new}' <<EOF

SV:123456:respawn:/usr/local/bin/svscanboot
EOF
        chmod 0644 /etc/inittab'{new}'
        mv -f /etc/inittab'{new}' /etc/inittab

        kill -HUP 1
        echo 'init should start svscan now.'
    fi

########################################
# /etc/rc.local (final fallback)

elif [[ -r /etc/rc.local ]]
then
    echo 'Found /etc/rc.local'

    if grep svscanboot /etc/rc.local >/dev/null
    then
        echo 'rc.local contains an svscanboot line. I assume that svscan is already running.'
    else
        echo 'Adding svscanboot to /etc/rc.local...'
        rm -f /etc/rc.local'{new}'

        cp /etc/rc.local /etc/rc.local'{new}'
        cat >> /etc/rc.local'{new}' <<EOF

csh -cf '/usr/local/bin/svscanboot &'
EOF
        chmod 0644 /etc/rc.local'{new}'
        mv -f /etc/rc.local'{new}' /etc/rc.local

        echo 'Reboot now to start svscan.'
    fi

########################################
# final option - give up

else
    echo 'Sorry, unable to make daemontools start when the system boots.'
fi

# end of post-install script
#
###############################################################################

%preun
###############################################################################
#
# pre-uninstall script

########################################
# stop all running services

for SDIR in /service/*
do
    if [[ -d $SDIR ]]
    then
        echo Shutting down $SDIR
        cd $SDIR
        rm $SDIR
        svc -dx .
        if [[ -d log ]]
        then
            echo Shutting down $SDIR/log
            svc -dx log
        fi
    fi
done

########################################
# if it's in upstart (either location), stop it and remove the upstart config

if [[ -f /etc/init/daemontools.conf ]]
then
    echo Found daemontools in upstart /etc/init
    /sbin/initctl stop daemontools
    rm -f /etc/init/daemontools.conf
fi

if [[ -f /etc/event.d/daemontools.conf ]]
then
    echo Found daemontools in upstart /etc/event.d
    /sbin/initctl stop daemontools
    rm -f /etc/event.d/daemontools.conf
fi

########################################
# if it's in inittab, remove it and send a HUP to init

if [[ -f /etc/inittab ]]
then
    if grep svscanboot /etc/inittab >/dev/null
    then
        echo 'Removing svscanboot from /etc/inittab'

        grep -v svscanboot /etc/inittab > /etc/inittab'{new}'
        chmod 0644 /etc/inittab'{new}'
        mv -f /etc/inittab'{new}' /etc/inittab
        kill -HUP 1
    fi
fi

########################################
# if it's in /etc/rc.local, remove it and try to manually kill it

if [[ -f /etc/rc.local ]]
then
    if grep svscanboot /etc/rc.local >/dev/null
    then
        echo 'Removing svscanboot from /etc/rc.local'

        grep -v svscanboot /etc/rc.local > /etc/rc.local'{new}'
        chmod 0644 /etc/rc.local'{new}'
        mv -f /etc/rc.local'{new}' /etc/rc.local

        X=`ps -o pid= -C svscanboot`
        if [[ -n "$X" ]]
        then
            echo "Killing svscanboot [$X]"
            kill $X
        fi

        X=`ps -o pid= -C svscan`
        if [[ -n "$X" ]]
        then
            echo "Killing svscan [$X]"
            kill $X
        fi
    fi
fi

# end of pre-uninstall script
#
###############################################################################

%files
/usr/local/bin/*

%changelog
* Thu Aug 29 2013 John Simpson <jms1@voalte.com>
- Added post-install and pre-uninstall scripts for upstart/init/rc.local

* Fri Feb 9 2007 Bruce Guenter <bruce@untroubled.org>
- Updated to version 0.76 finally.

* Thu May 5 2001 Bruce Guenter <bruceg@em.ca>
- Added a hack to allow building on glibc 2.2 systems (such as RedHat 7.x)

* Fri Dec 1 2000 Bruce Guenter <bruceg@em.ca>
- Reverted to a completely plain, unmodified install.

* Mon Mar 6 2000 Bruce Guenter <bruceg@em.ca>
- Updated to version 0.70.

* Mon Dec 6 1999 Bruce Guenter <bruceg@em.ca>
- Modified svscan init script to start all services as "down" on
  startup, and to clean up properly on exit.
- Removed /var/lock/svc directory.
